@@ -29,6 +29,7 @@ extern "C" {
 #include <stdint.h>
 #include <glib/gtypes.h>
 #include <gisi/modem.h>
+#include "phonet.h"
 
 struct _GIsiClient;
 typedef struct _GIsiClient GIsiClient;
@@ -52,12 +53,18 @@ GIsiClient *g_isi_client_create(GIsiModem *modem, uint8_t resource);
 GIsiRequest *g_isi_verify(GIsiClient *client, GIsiVerifyFunc func,
 				void *opaque);
 
+GIsiRequest *g_isi_verify_resource(GIsiClient *client, uint8_t resource,
+					GIsiVerifyFunc func, void *opaque);
+
 uint8_t g_isi_client_resource(GIsiClient *client);
 
 void g_isi_version_set(GIsiClient *client, int major, int minor);
 int g_isi_version_major(GIsiClient *client);
 int g_isi_version_minor(GIsiClient *client);
 
+void g_isi_server_object_set(GIsiClient *client, uint16_t obj);
+uint8_t g_isi_server_object(GIsiClient *client);
+
 void g_isi_client_set_debug(GIsiClient *client, GIsiDebugFunc func,
 				void *opaque);
 
@@ -73,11 +80,25 @@ GIsiRequest *g_isi_request_vmake(GIsiClient *client, const struct iovec *iov,
 					size_t iovlen, unsigned timeout,
 					GIsiResponseFunc func, void *opaque);
 
+GIsiRequest *g_isi_sendto(GIsiClient *client,
+				struct sockaddr_pn *dst,
+				const void *data, size_t len,
+				unsigned timeout,
+				GIsiResponseFunc func, void *opaque,
+				GDestroyNotify notify);
+
 GIsiRequest *g_isi_send(GIsiClient *client, const void *data, size_t len,
 			unsigned timeout,
 			GIsiResponseFunc func, void *opaque,
 			GDestroyNotify notify);
 
+GIsiRequest *g_isi_vsendto(GIsiClient *client,
+				struct sockaddr_pn *dst,
+				const struct iovec *iov, size_t iovlen,
+				unsigned timeout,
+				GIsiResponseFunc func, void *opaque,
+				GDestroyNotify notify);
+
 GIsiRequest *g_isi_vsend(GIsiClient *client,
 				const struct iovec *iov, size_t iovlen,
 				unsigned timeout,
@@ -86,9 +107,13 @@ GIsiRequest *g_isi_vsend(GIsiClient *client,
 
 void g_isi_request_cancel(GIsiRequest *req);
 
+int g_isi_commit_subscriptions(GIsiClient *client);
+int g_isi_add_subscription(GIsiClient *client, uint8_t res, uint8_t type,
+				GIsiIndicationFunc cb, void *data);
+void g_isi_remove_subscription(GIsiClient *client, uint8_t res, uint8_t type);
+
 int g_isi_subscribe(GIsiClient *client, uint8_t type,
 			GIsiIndicationFunc func, void *opaque);
-
 void g_isi_unsubscribe(GIsiClient *client, uint8_t type);
 
 #ifdef __cplusplus
