@@ -1,4 +1,5 @@
 /*
+ *
  *  oFono - Open Source Telephony
  *
  *  Copyright (C) 2008-2010  Intel Corporation. All rights reserved.
@@ -58,8 +59,8 @@
 
 #include <drivers/atmodem/vendor.h>
 
-#define CALYPSO_POWER_PATH "/sys/bus/platform/devices/neo1973-pm-gsm.0/power_on"
-#define CALYPSO_RESET_PATH "/sys/bus/platform/devices/neo1973-pm-gsm.0/reset"
+#define CALYPSO_POWER_PATH "/sys/bus/platform/devices/gta02-pm-gsm.0/power_on"
+#define CALYPSO_RESET_PATH "/sys/bus/platform/devices/gta02-pm-gsm.0/reset"
 
 enum powercycle_state {
 	POWERCYCLE_STATE_POWER0 = 0,
@@ -71,11 +72,13 @@ enum powercycle_state {
 
 #define NUM_DLC 4
 
-#define VOICE_DLC 0
-#define NETREG_DLC 1
-#define SMS_DLC 2
-#define AUX_DLC 3
-#define SETUP_DLC 3
+#define VOICE_DLC   0
+#define NETREG_DLC  1
+#define SMS_DLC     2
+#define AUX_DLC     3
+#define SETUP_DLC   3
+
+static char *debug_prefixes[NUM_DLC] = { "Voice: ", "Net: ", "SMS: ", "Aux: " };
 
 struct calypso_data {
 	GAtMux *mux;
@@ -90,21 +93,11 @@ struct calypso_data {
 static const char *cpin_prefix[] = { "+CPIN:", NULL };
 static const char *none_prefix[] = { NULL };
 
-static void calypso_debug(const char *str, void *data)
+static void calypso_debug(const char *str, void *user_data)
 {
-	guint dlc = GPOINTER_TO_UINT(data);
+	const char *prefix = user_data;
 
-	ofono_info("DLC%u: %s", dlc, str);
-}
-
-static void calypso_mux_debug(const char *str, void *data)
-{
-	ofono_info("MUX: %s", str);
-}
-
-static void calypso_setup_debug(const char *str, void *data)
-{
-	ofono_info("Setup: %s", str);
+	ofono_info("%s%s", prefix, str);
 }
 
 static int calypso_probe(struct ofono_modem *modem)
@@ -112,7 +105,7 @@ static int calypso_probe(struct ofono_modem *modem)
 	const char *device;
 	struct calypso_data *data;
 
-	DBG("");
+	DBG("%p", modem);
 
 	device = ofono_modem_get_string(modem, "Device");
 	if (device == NULL)
@@ -131,7 +124,7 @@ static void calypso_remove(struct ofono_modem *modem)
 {
 	struct calypso_data *data = ofono_modem_get_data(modem);
 
-	DBG("");
+	DBG("%p", modem);
 
 	g_free(data);
 }
@@ -272,7 +265,7 @@ static void mux_setup(GAtMux *mux, gpointer user_data)
 	data->mux = mux;
 
 	if (getenv("OFONO_AT_DEBUG"))
-		g_at_mux_set_debug(data->mux, calypso_mux_debug, NULL);
+		g_at_mux_set_debug(data->mux, calypso_debug, "MUX: ");
 
 	g_at_mux_start(mux);
 
@@ -286,7 +279,7 @@ static void mux_setup(GAtMux *mux, gpointer user_data)
 
 		if (getenv("OFONO_AT_DEBUG"))
 			g_at_chat_set_debug(data->dlcs[i], calypso_debug,
-						GUINT_TO_POINTER(i));
+							debug_prefixes[i]);
 
 		g_at_chat_set_wakeup_command(data->dlcs[i], "AT\r", 500, 5000);
 	}
@@ -336,7 +329,7 @@ static void modem_initialize(struct ofono_modem *modem)
 		goto error;
 
 	if (getenv("OFONO_AT_DEBUG") != NULL)
-		g_at_chat_set_debug(chat, calypso_setup_debug, NULL);
+		g_at_chat_set_debug(chat, calypso_debug, "Setup: ");
 
 	g_at_chat_set_wakeup_command(chat, "AT\r", 500, 5000);
 
@@ -428,6 +421,8 @@ static int calypso_enable(struct ofono_modem *modem)
 {
 	struct calypso_data *data = ofono_modem_get_data(modem);
 
+	DBG("%p", modem);
+
 	if (write_file(CALYPSO_POWER_PATH, FALSE) == FALSE)
 		return -EINVAL;
 
@@ -442,7 +437,7 @@ static int calypso_disable(struct ofono_modem *modem)
 	struct calypso_data *data = ofono_modem_get_data(modem);
 	int i;
 
-	DBG("");
+	DBG("%p", modem);
 
 	for (i = 0; i < NUM_DLC; i++) {
 		g_at_chat_unref(data->dlcs[i]);
@@ -466,7 +461,7 @@ static void calypso_pre_sim(struct ofono_modem *modem)
 {
 	struct calypso_data *data = ofono_modem_get_data(modem);
 
-	DBG("");
+	DBG("%p", modem);
 
 	ofono_devinfo_create(modem, 0, "atmodem", data->dlcs[AUX_DLC]);
 	data->sim = ofono_sim_create(modem, 0, "atmodem", data->dlcs[AUX_DLC]);
@@ -530,7 +525,7 @@ static void calypso_post_sim(struct ofono_modem *modem)
 	struct calypso_data *data = ofono_modem_get_data(modem);
 	struct ofono_message_waiting *mw;
 
-	DBG("");
+	DBG("%p", modem);
 
 	ofono_ussd_create(modem, 0, "atmodem", data->dlcs[AUX_DLC]);
 	ofono_call_forwarding_create(modem, 0, "atmodem", data->dlcs[AUX_DLC]);
